using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

using Microsoft.Devices.Sensors;

using System.IO;
using System.IO.IsolatedStorage;

using Microsoft.Phone.Shell;

namespace CheeseLander
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CheeseLanderGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        SpriteFont font;

        int displayWidth;
        int displayHeight;

        // Cheese 
        Texture2D cheeseTexture;
        Rectangle cheeseRectangle;
        int cheeseWidth;
        float cheeseWidthFactor = 8;
        float frictionFactor = 0.99f;

        Vector2 cheesePosition;
        Vector2 cheeseSpeed;
        Vector2 cheeseAcceleration;

        float cheeseMaxLandSpeed = 0.8f;

        // Bread

        Texture2D breadTexture;
        Rectangle breadRectangle;
        int breadWidth;
        float breadWidthFactor = 3;

        // Starfield background
        Texture2D backgroundTexture;

        // Draw rectangle for messages and backgrounds
        Rectangle fullScreenDrawPos;


        // Message textures

        Texture2D titleTexture;
        Texture2D wonTexture;
        Texture2D lostTexture;
        Texture2D pausedTexture;

        // Accelleration values

        Accelerometer accel;
        Vector3 accelReading;
        float acceleratorPower = 0.1f;

        // Time counter
        int gameTicks = 0;

        int fastestTicks = 300000000;


        enum gameState
        {
            title,
            playing,
            paused,
            won,
            lost
        }

        private gameState state = gameState.title;

        SoundEffect wonSound;
        SoundEffect lostSound;
        SoundEffect engineSound;
        SoundEffectInstance engineSoundInstance;


        private void bindEvents()
        {
            PhoneApplicationService.Current.Activated += new EventHandler<ActivatedEventArgs>(Current_Activated);
            PhoneApplicationService.Current.Deactivated += new EventHandler<DeactivatedEventArgs>(Current_Deactivated);
            PhoneApplicationService.Current.Launching += new EventHandler<LaunchingEventArgs>(Current_Launching);
            PhoneApplicationService.Current.Closing += new EventHandler<ClosingEventArgs>(Current_Closing);

            //Guide.IsScreenSaverEnabled = false;
            graphics.IsFullScreen = true;

            this.Activated += new EventHandler<EventArgs>(EventLogGame_Activated);
            this.Deactivated += new EventHandler<EventArgs>(EventLogGame_Deactivated);
        }

        void EventLogGame_Deactivated(object sender, EventArgs e)
        {
            // called when we get an incoming event such as a phone call
            // pause the game
            pauseGame();
        }

        void EventLogGame_Activated(object sender, EventArgs e)
        {
            // called when the incoming event is complete
            // could resume the game here - but don't
        }

        void Current_Closing(object sender, ClosingEventArgs e)
        {
            // called when the game is closing
            // there is no status to save so we do 
            // nothing
        }

        void Current_Launching(object sender, LaunchingEventArgs e)
        {
            // called when the game is launching
            // Load the game from isolated storage
            if (!loadGameFromIsolatedStorage())
            {
                selectTitle();
            }
        }

        void Current_Deactivated(object sender, DeactivatedEventArgs e)
        {
            // called when the game is tombstoned
            // Store the game in the state object
            saveGameToIsolatedStorage();
            //saveGameToStateObject();
        }

        void Current_Activated(object sender, ActivatedEventArgs e)
        {
            // Called when the game returns from a tombstone
            // restore the game from the state object
            if (!loadGameFromIsolatedStorage())
            {
                selectTitle();
            }
        }

        public CheeseLanderGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = 240;
            graphics.PreferredBackBufferHeight = 400;

            bindEvents();

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
accel = new Accelerometer();

accel.ReadingChanged += new EventHandler<AccelerometerReadingEventArgs>(accel_ReadingChanged);

accel.Start();

            base.Initialize();
        }

        object accelLock = new object();

        void accel_ReadingChanged(object sender, AccelerometerReadingEventArgs e)
        {
            lock (accelLock)
            {
                accelReading.X = (float)e.X;
                accelReading.Y = (float)e.Y;
                accelReading.Z = (float)e.Z;
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            font = Content.Load<SpriteFont>("MessageFont");

            displayWidth = GraphicsDevice.Viewport.Width;
            displayHeight = GraphicsDevice.Viewport.Height;

            cheeseWidth = (int)(displayWidth / cheeseWidthFactor);
            breadWidth = (int)(displayWidth / breadWidthFactor);

            // Create the draw rectangle 
            // Cheese should be a 12th the size of the screen
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the cheese and set the size of the draw rectangle
            cheeseTexture = Content.Load<Texture2D>("cheese");
            cheeseRectangle.Width = cheeseWidth;
            float aspectRatio = (float)cheeseTexture.Width / cheeseTexture.Height;
            cheeseRectangle.Height = (int)((cheeseWidth / aspectRatio) + 0.5f);

            // Load the bread and set the size of the draw rectangle
            breadTexture = Content.Load<Texture2D>("bread");
            breadRectangle.Width = breadWidth;
            aspectRatio = (float)breadTexture.Width / breadTexture.Height;
            breadRectangle.Height = (int)((breadWidth / aspectRatio) + 0.5f);

            // Load the background
            backgroundTexture = Content.Load<Texture2D>("background");

            // Load the win and lose textures
            wonTexture = Content.Load<Texture2D>("wonScreen");
            lostTexture = Content.Load<Texture2D>("lostScreen");
            titleTexture = Content.Load<Texture2D>("landerTitle");
            pausedTexture = Content.Load<Texture2D>("pausedScreen");

            fullScreenDrawPos = new Rectangle(
                0, 0,
                displayWidth, displayHeight);

            // Load the sound effects
            wonSound = Content.Load<SoundEffect>("wonSound");
            lostSound = Content.Load<SoundEffect>("lostSound");
            engineSound = Content.Load<SoundEffect>("engineSound");

            // create an instance of the engine sound 
            engineSoundInstance = engineSound.CreateInstance();
            engineSoundInstance.Volume = 0;
            engineSoundInstance.IsLooped = true;

            saveGameToStateObject();

            loadGameFromStateObject();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        float engineSoundThreshold = 0.07f;
        float enginePitchFactor = 20.0f;

        private void stopEngineSound()
        {
            if (engineSoundInstance.State == SoundState.Playing)
            {
                engineSoundInstance.Pause();
            }
        }

        private void startEngineSound()
        {
            switch (engineSoundInstance.State)
            {
                case SoundState.Paused:
                    engineSoundInstance.Resume();
                    break;
                case SoundState.Stopped:
                    engineSoundInstance.Play();
                    break;
            }
        }

        private void setEngineSound(float control)
        {
            engineSoundInstance.Volume = Math.Min(control / engineSoundThreshold, 1);
            engineSoundInstance.Pitch = Math.Min(control * enginePitchFactor, 1);
        }

        string timeMessage="";

        private void updateGame()
        {
            // Check for pause

            GamePadState pad = GamePad.GetState(PlayerIndex.One);

            if (pad.Buttons.Back == ButtonState.Pressed)
            {
                pauseGame();
            }

            // Update the X and Y positions

            // Get the acceleration vector

            lock (accelLock)
            {
                cheeseAcceleration.X = accelReading.X * acceleratorPower;
                cheeseAcceleration.Y = -accelReading.Y * acceleratorPower;
            }

            setEngineSound(cheeseAcceleration.Length());

            // Update the speed
            cheeseSpeed = Vector2.Add(cheeseSpeed, cheeseAcceleration);
            cheeseSpeed = Vector2.Multiply(cheeseSpeed, frictionFactor);

            cheesePosition = Vector2.Add(cheesePosition, cheeseSpeed);

            // Set the cheese Draw rectangle to cheeseX and cheeseY
            cheeseRectangle.X = (int)(cheesePosition.X + 0.5f);
            cheeseRectangle.Y = (int)(cheesePosition.Y + 0.5f);

            // Limit the sides

            if (cheesePosition.X < 0)
            {
                cheeseSpeed.X = (float)Math.Abs(cheeseSpeed.X);
            }

            if (cheesePosition.Y < 0)
            {
                cheeseSpeed.Y = (float)Math.Abs(cheeseSpeed.Y);
            }

            if (cheeseRectangle.Right > displayWidth)
            {
                cheeseSpeed.X = -1 * (float)Math.Abs(cheeseSpeed.X);
            }

            if (cheeseRectangle.Bottom > GraphicsDevice.Viewport.Height)
            {
                cheeseSpeed.Y = -1 * (float)Math.Abs(cheeseSpeed.Y);
            }

            if (cheeseRectangle.Intersects(breadRectangle))
            {
                // May have won 
                if (cheeseRectangle.Top < breadRectangle.Top &&
                    cheeseRectangle.Left >= breadRectangle.Left &&
                    cheeseRectangle.Right <= breadRectangle.Right &&
                    cheeseSpeed.Length() < cheeseMaxLandSpeed)
                {
                    // cheese is properly landed  player wins
                    gameWon();
                }
                else
                {
                    // cheese is badly landed  player loses
                    gameLost();
                }
            }

            gameTicks++;

            timeMessage = (gameTicks / 30).ToString();
        }

        Vector3 accelState = Vector3.Zero;

        void acc_ReadingChanged(object sender, AccelerometerReadingEventArgs e)
        {
            accelState.X = (float)e.X;
            accelState.Y = (float)e.Y;
            accelState.Z = (float)e.Z;
        }

        private void gameWon()
        {
            stopEngineSound();
            wonSound.Play();
            state = gameState.won;
        }

        private void gameLost()
        {
            stopEngineSound();
            lostSound.Play();
            state = gameState.lost;
        }

        Random rand = new Random();

        private void newGame()
        {
            // Put the cheese at a random position on the top of the scren
            cheesePosition.X = rand.Next(displayWidth - cheeseWidth);
            cheesePosition.Y = 0;

            cheeseSpeed = Vector2.Zero;

            // Move the bread to a new position across the screen
            breadRectangle.X = rand.Next(displayWidth - breadWidth);
            breadRectangle.Y = displayHeight - breadRectangle.Height;

            // Start playing the game
            state = gameState.playing;

            // Start the engine
            startEngineSound();
        }

        private void selectTitle()
        {
            state = gameState.title;
        }

        private void pauseGame()
        {
            stopEngineSound();
            state = gameState.paused;
        }

        private void resumeGame()
        {
            startEngineSound();
            state = gameState.playing;
        }

        private void checkNewGame()
        {
            // If back is pressed on the new game screens the game ends

            GamePadState pad = GamePad.GetState(PlayerIndex.One);

            if (pad.Buttons.Back == ButtonState.Pressed)
            {
                Exit();
            }
            // Get the Touch Screen

            TouchCollection touches = TouchPanel.GetState();

            foreach (TouchLocation location in touches)
            {
                if (location.State == TouchLocationState.Pressed)
                {
                    newGame();
                    break;
                }
            }
        }

        private void checkPausedGame()
        {
            GamePadState pad = GamePad.GetState(PlayerIndex.One);

            if (pad.Buttons.Back == ButtonState.Pressed)
            {
                resumeGame();
            }
        }

        string filename = "lander.dat";

        private void saveGameToStateObject()
        {
            PhoneApplicationService.Current.State.Add("State", state);
            PhoneApplicationService.Current.State.Add("CheeseX", cheesePosition.X);
            PhoneApplicationService.Current.State.Add("CheeseY", cheesePosition.Y);
            PhoneApplicationService.Current.State.Add("CheeseXSpeed", cheeseSpeed.X);
            PhoneApplicationService.Current.State.Add("CheeseYSpeed", cheeseSpeed.Y);
            PhoneApplicationService.Current.State.Add("BreadX", breadRectangle.X);
            PhoneApplicationService.Current.State.Add("BreadY", breadRectangle.Y);
            PhoneApplicationService.Current.State.Add("Ticks", gameTicks);
        }

        private bool loadGameFromStateObject()
        {
            bool loadedOK = true;

            if (PhoneApplicationService.Current.State.ContainsKey("State"))
            {
                state = (gameState)PhoneApplicationService.Current.State["State"];
                PhoneApplicationService.Current.State.Remove("State");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("CheeseX"))
            {
                cheesePosition.X = (float)PhoneApplicationService.Current.State["CheeseX"];
                PhoneApplicationService.Current.State.Remove("CheeseX");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("CheeseY"))
            {
                cheesePosition.Y = (float)PhoneApplicationService.Current.State["CheeseY"];
                PhoneApplicationService.Current.State.Remove("CheeseY");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("CheeseXSpeed"))
            {
                cheeseSpeed.X = (float)PhoneApplicationService.Current.State["CheeseXSpeed"];
                PhoneApplicationService.Current.State.Remove("CheeseXSpeed");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("CheeseYSpeed"))
            {
                cheeseSpeed.Y = (float)PhoneApplicationService.Current.State["CheeseYSpeed"];
                PhoneApplicationService.Current.State.Remove("CheeseYSpeed");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("BreadX"))
            {
                breadRectangle.X = (int)PhoneApplicationService.Current.State["BreadX"];
                PhoneApplicationService.Current.State.Remove("BreadX");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("BreadY"))
            {
                breadRectangle.Y = (int)PhoneApplicationService.Current.State["BreadY"];
                PhoneApplicationService.Current.State.Remove("BreadY");
            }
            else
                loadedOK = false;

            if (PhoneApplicationService.Current.State.ContainsKey("Ticks"))
            {
                gameTicks = (int)PhoneApplicationService.Current.State["Ticks"];
                PhoneApplicationService.Current.State.Remove("Ticks");
            }
            else
                loadedOK = false;

            return loadedOK;
        }

        private void saveGameToIsolatedStorage()
        {

            using (IsolatedStorageFile isf =
                          IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream rawStream = isf.CreateFile(filename))
                {
                    StreamWriter writer = new StreamWriter(rawStream);
                    int stateInt = (int)state; // convert state into an integer
                    writer.WriteLine(stateInt); // save the state
                    writer.WriteLine(cheesePosition.X);
                    writer.WriteLine(cheesePosition.Y);
                    writer.WriteLine(cheeseSpeed.X);
                    writer.WriteLine(cheeseSpeed.Y);
                    writer.WriteLine(breadRectangle.X);
                    writer.WriteLine(breadRectangle.Y);
                    writer.WriteLine(gameTicks);
                    writer.Close();
                }
            }
        }

        private bool loadGameFromIsolatedStorage()
        {
            bool loadedOK = true;

            using (IsolatedStorageFile isf =
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isf.FileExists(filename))
                {
                    try
                    {
                        using (IsolatedStorageFileStream rawStream =
                            isf.OpenFile(filename, System.IO.FileMode.Open))
                        {
                            StreamReader reader = new StreamReader(rawStream);
                            int stateInt = int.Parse(reader.ReadLine());
                            state = (gameState)stateInt; // convert into a state
                            cheesePosition.X = float.Parse(reader.ReadLine());
                            cheesePosition.Y = float.Parse(reader.ReadLine());
                            cheeseSpeed.X = float.Parse(reader.ReadLine());
                            cheeseSpeed.Y = float.Parse(reader.ReadLine());
                            breadRectangle.X = int.Parse(reader.ReadLine());
                            breadRectangle.Y = int.Parse(reader.ReadLine());
                            gameTicks = int.Parse(reader.ReadLine());
                            reader.Close();
                        }
                    }
                    catch
                    {
                        loadedOK = false;
                    }
                    // Remove the file so that it isn't used next time
                    isf.DeleteFile(filename);
                }
                else
                {
                    loadedOK = false;
                }
            }
            return loadedOK;
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            switch (state)
            {
                case gameState.title:
                    checkNewGame();
                    break;
                case gameState.playing:
                    updateGame();
                    break;
                case gameState.paused:
                    checkPausedGame();
                    break;
                case gameState.lost:
                    checkNewGame();
                    break;
                case gameState.won:
                    checkNewGame();
                    break;
            }

            base.Update(gameTime);
        }

        private void drawString(string message)
        {
            int xPos = (int)((displayWidth - font.MeasureString(message).X) / 2);
            spriteBatch.DrawString(font, message, new Vector2(xPos, 0), Color.White);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            switch (state)
            {
                case gameState.title:
                    spriteBatch.Draw(titleTexture, fullScreenDrawPos, Color.White);
                    break;

                case gameState.playing:
                    spriteBatch.Draw(backgroundTexture, fullScreenDrawPos, Color.White);
                    spriteBatch.Draw(breadTexture, breadRectangle, Color.White);
                    spriteBatch.Draw(cheeseTexture, cheeseRectangle, Color.White);
                    drawString(timeMessage);
                    break;

                case gameState.paused:
                    spriteBatch.Draw(pausedTexture, fullScreenDrawPos, Color.White);
                    break;

                case gameState.won:
                    spriteBatch.Draw(wonTexture, fullScreenDrawPos, Color.White);
                    break;

                case gameState.lost:
                    spriteBatch.Draw(lostTexture, fullScreenDrawPos, Color.White);
                    break;
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
